﻿Public Class ScaledBitmap
   Private _Size As Size ' Größe in Pixel
   Private _ScaledSize As SizeF ' Größe in sklalierten Koordinaten
   Private _Origin As PointF ' Koordinaten der linken unteren Ecke in sklalierten Koordinaten
   Private _Bitmap As Bitmap
   Private _TransformMatrix As New Drawing2D.Matrix
   Private _Graphics As Graphics

   ''' <summary>
   ''' Gibt an, wie ein Objekt in Bezug zu seinem Ursprung ausgerichtet wird.
   ''' </summary>
   Public Enum ObjectAlignment
      ''' <summary>Das Objekt wird mit der Standardausrichtung ausgegeben.</summary>
      Standard
      ''' <summary>Das Objekt wird zentriert.</summary>
      Center
   End Enum

   ''' <summary>
   ''' Initialisiert eine neue Instanz der <see cref="ScaledBitmap"/>-Klasse.
   ''' </summary>
   ''' <param name="Size">Größe der Bitmap in Pixel.</param>
   ''' <param name="ScaledSize">Größe in skalierten Koordinaten.</param>
   ''' <param name="Origin">Skalierte Koordinaten der linken unteren Ecke.</param>
   Public Sub New(Size As Size, ScaledSize As SizeF, Origin As PointF)
      _Size = Size
      _ScaledSize = ScaledSize
      _Origin = Origin
      _Bitmap = New Bitmap(_Size.Width, _Size.Height)
      ComputeTransformMatrix()
      _Graphics = Graphics.FromImage(_Bitmap)
   End Sub

   ''' <summary>
   ''' Liefert die aktuell erstellte Bitmap.
   ''' </summary>
   ''' <returns>Die aktuell erstellte Bitmap.</returns>
   Public Function getBitmap() As Bitmap
      Return _Bitmap
   End Function

   ''' <summary>
   ''' Berechnet die Tranformationsmatrix
   ''' </summary>
   Private Sub ComputeTransformMatrix()
      With _Bitmap
         Dim mx = .Width / _ScaledSize.Width ' Skalierung X
         Dim my = - .Height / _ScaledSize.Height ' Skalierung Y

         _TransformMatrix = New Drawing2D.Matrix
         _TransformMatrix.Translate(-_Origin.X, -_Origin.Y) ' Skalierte linke untere Ecke auf skaliert (0 | 0)
         _TransformMatrix.Scale(mx, my, Drawing2D.MatrixOrder.Append) ' Skalierung auf Control-Größe
         _TransformMatrix.Translate(0, .Height - 1, Drawing2D.MatrixOrder.Append) ' Bezugspunkt ist links unten (anstatt links oben)
      End With
   End Sub

#Region " Transformationen "
   ''' <summary>
   ''' Erstellt aus den angegeben skalierten Koordinaten ein <see cref="PointF"/>-Objekt in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="x">X-Koordinate in skalierten Koordinaten</param>
   ''' <param name="y">Y-Koordinate in skalierten Koordinaten</param>
   ''' <returns>Punkt in Pixel-Koordinaten.</returns>
   Public Function Transform(x As Single, y As Single) As PointF
      Dim pts As PointF() = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Return pts(0)
   End Function

   ''' <summary>
   ''' Erstellt aus den angegeben skalierten Koordinaten ein <see cref="Point"/>-Objekt in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="x">X-Koordinate in skalierten Koordinaten</param>
   ''' <param name="y">Y-Koordinate in skalierten Koordinaten</param>
   ''' <returns>Punkt in Pixel-Koordinaten.</returns>
   Public Function Transform(x As Integer, y As Integer) As Point
      Dim pts As Point() = {New Point(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Return pts(0)
   End Function

   ''' <summary>
   ''' Erstellt aus den angegeben skalierten Koordinaten ein <see cref="PointF"/>-Objekt in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="x">X-Koordinate in skalierten Koordinaten</param>
   ''' <param name="y">Y-Koordinate in skalierten Koordinaten</param>
   ''' <returns>Punkt in Pixel-Koordinaten.</returns>
   Public Function Transform(x As Double, y As Double) As PointF
      Return Transform(CSng(x), CSng(y))
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="PointF"/>-Objekt in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="pt">Punkt in skalierten Koordinaten</param>
   ''' <returns>Punkt in Pixel-Koordinaten.</returns>
   Public Function Transform(pt As PointF) As PointF
      Dim pts As PointF() = {pt}
      _TransformMatrix.TransformPoints(pts)
      Return pts(0)
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="Point"/>-Objekt in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="pt">Punkt in skalierten Koordinaten</param>
   ''' <returns>Punkt in Pixel-Koordinaten.</returns>
   Public Function Transform(pt As Point) As Point
      Dim pts As Point() = {pt}
      _TransformMatrix.TransformPoints(pts)
      Return pts(0)
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="PointF"/>-Array in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="pts">Punkte-Array in skalierten Koordinaten</param>
   ''' <returns>Punkte-Array in Pixel-Koordinaten.</returns>
   Public Function Transform(pts As PointF()) As PointF()
      Dim pts1 = CType(pts.Clone, PointF()).ToArray
      _TransformMatrix.TransformPoints(pts1)
      Return pts1
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="Point"/>-Array in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="pts">Punkte-Array in skalierten Koordinaten</param>
   ''' <returns>Punkte-Array in Pixel-Koordinaten.</returns>
   Public Function Transform(pts As Point()) As Point()
      Dim pts1 = CType(pts.Clone, Point()).ToArray
      _TransformMatrix.TransformPoints(pts1)
      Return pts1
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="RectangleF"/>-Objekt in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="rct">Rechteck in skalierten Koordinaten</param>
   ''' <returns>Rechteck in Pixel-Koordinaten.</returns>
   Public Function Transform(rct As RectangleF) As RectangleF
      Dim pts() As PointF = {rct.Location, New PointF(rct.Right, rct.Bottom)}
      _TransformMatrix.TransformPoints(pts)
      Return New RectangleF(pts(0).X, pts(0).Y, pts(1).X - pts(0).X, pts(0).Y - pts(1).Y)
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="Rectangle"/>-Objekt in skalierten Koordinaten eines in Pixel-Koordinaten.
   ''' </summary>
   ''' <param name="rct">Rechteck in skalierten Koordinaten.</param>
   ''' <returns>Rechteck in Pixel-Koordinaten.</returns>
   Public Function Transform(rct As Rectangle) As Rectangle
      Dim pts() As Point = {rct.Location, New Point(rct.Right, rct.Bottom)}
      _TransformMatrix.TransformPoints(pts)
      Return New Rectangle(pts(0).X, pts(0).Y, pts(1).X - pts(0).X, pts(0).Y - pts(1).Y)
   End Function

   ''' <summary>
   ''' Erstellt aus eimen <see cref="Drawing.Size"/>-Object in Pixel-Koordinaten eines in skalierten Koordinaten.
   ''' </summary>
   ''' <param name="Size"><see cref="Drawing.Size"/>-Objekt in skalierten Koordinaten.</param>
   ''' <returns><see cref="Drawing.Size"/>-Objekt in Pixel Koordinaten</returns>
   Public Function Transform(Size As Size) As Size
      Dim p As New Rectangle(0, 0, Size.Width, Size.Height)
      p = Transform(p)
      Return New Size(p.Width, p.Height)
   End Function

   ''' <summary>
   ''' Erstellt aus eimen <see cref="SizeF"/>-Object in Pixel-Koordinaten eines in skalierten Koordinaten.
   ''' </summary>
   ''' <param name="Size"><see cref="SizeF"/>-Objekt in skalierten Koordinaten.</param>
   ''' <returns><see cref="SizeF"/>-Objekt in Pixel Koordinaten</returns>
   Public Function Transform(Size As SizeF) As SizeF
      Dim p As New RectangleF(0, 0, Size.Width, Size.Height)
      p = Transform(p)
      Return New SizeF(p.Width, p.Height)
   End Function

   ''' <summary>
   ''' Transformiert eine horizontale Länge (Weite) in skalierten Werte in eine solche mit Pixel-Werten.</summary>
   ''' <param name="ScaledWidth">Horizontale Länge in skalierten Werten.</param>
   ''' <returns>Die horizontale Länge (Weite) umgerechent in Pixel-Werte.</returns>
   ''' <remarks>Horizontale Längen (Weiten) und vertikale Längen (Höhen) haben 
   ''' unterschiedliche Skalierungsfaktoren.</remarks>
   Public Function TransformScaledWidthToPixelWidth(ScaledWidth As Single) As Single
      Return ScaledWidth / _ScaledSize.Width * _Size.Width
   End Function

   ''' <summary>
   ''' Transformiert eine vertikale Länge (Höhe) in skalierten Werte in eine solche mit Pixel-Werten.</summary>
   ''' <param name="ScaledHeight">Vertikale Länge in skalierten Werten.</param>
   ''' <returns>Die vertikale Länge (Weite) umgerechent in Pixel-Werte.</returns>
   ''' <remarks>Horizontale Längen (Weiten) und vertikale Längen (Höhen) haben 
   ''' unterschiedliche Skalierungsfaktoren.</remarks>
   Public Function TransformScaledHeightToPixelHeight(ScaledHeight As Single) As Single
      Return ScaledHeight / _ScaledSize.Height * _Size.Height
   End Function

   ''' <summary>
   ''' Transformiert eine horizontale Länge (Weite) in Pixel in eine solche mit skalierten Werten.</summary>
   ''' <param name="PixelWidth">Horizontale Länge in Pixel.</param>
   ''' <returns>Die horizontale Länge (Weite) umgerechent in skalierte Werte.</returns>
   ''' <remarks>Horizontale Längen (Weiten) und vertikale Längen (Höhen) haben 
   ''' unterschiedliche Skalierungsfaktoren.</remarks>
   Public Function TransformPixelWidthToScaledWith(PixelWidth As Single) As Single
      Return PixelWidth / _Size.Width * _ScaledSize.Width
   End Function

   ''' <summary>
   ''' Transformiert eine vertikale Länge (Höhe) in Pixel in eine solche mit skalierten Werten.</summary>
   ''' <param name="PixelHeight">Vertikale Länge in Pixel.</param>
   ''' <returns>Die vertikale Länge (Höhe) umgerechent in skalierte Werte.</returns>       
   ''' <remarks>Horizontale Längen (Weiten) und vertikale Längen (Höhen) haben 
   ''' unterschiedliche Skalierungsfaktoren.</remarks>
   Public Function TransformPixelHeightToScaledHeight(PixelHeight As Single) As Single
      Return PixelHeight / _Size.Height * _ScaledSize.Height
   End Function

   ''' <summary>
   ''' Erstellt aus einem <see cref="PointF"/>-Objekt in Pixel-Koordinaten eines in skalierten Koordinaten.
   ''' </summary>
   ''' <param name="pt">Punkt in Pixel-Koordinaten</param>
   ''' <returns>Punkt in skalierten Koordinaten.</returns>
   Public Function InversTransform(pt As PointF) As PointF
      Dim pts As PointF() = {pt}
      Dim t = _TransformMatrix.Clone
      t.Invert()
      t.TransformPoints(pts)
      Return pts(0)
   End Function


#End Region

#Region " Draw-Methoden "
#Region " DrawString "
   ''' <summary>
   ''' Zeichnet die angegebene Textzeichenfolge an der angegebenen Position mit dem angegebenen <see cref="Brush"/>-Objekt 
   ''' und dem angegebenen <see cref="Font"/>-Objekt.
   ''' </summary>
   ''' <param name="Text">Zu zeichnende Zeichenfolge.</param>
   ''' <param name="Font"><see cref="Font"/>, die das Textformat der Zeichenfolge definiert.</param>
   ''' <param name="Brush"><see cref="Brush"/>, der die Farbe und Struktur des gezeichneten Texts bestimmt.</param>
   ''' <param name="Position">Die skalierte Position der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <remarks>Die Textgröße wird nicht skaliert.</remarks>
   Public Sub DrawString(Text As String, Font As Font, Brush As Brush, Position As PointF)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {Position}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawString(Text, Font, Brush, pts(0))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet die angegebene Textzeichenfolge an der angegebenen Position mit dem angegebenen <see cref="Brush"/>-Objekt 
   ''' und dem angegebenen <see cref="Font"/>-Objekt.
   ''' </summary>
   ''' <param name="Text">Zu zeichnende Zeichenfolge.</param>
   ''' <param name="Font"><see cref="Font"/>, die das Textformat der Zeichenfolge definiert.</param>
   ''' <param name="Brush"><see cref="Brush"/>, der die Farbe und Struktur des gezeichneten Texts bestimmt.</param>
   ''' <param name="Position">Die skalierte Position der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <remarks>Die Textgröße wird nicht skaliert.</remarks>
   Public Sub DrawString(Text As String, Font As Font, Brush As Brush, Position As Point)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {Position}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawString(Text, Font, Brush, pts(0))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet die angegebene Textzeichenfolge an der angegebenen Position mit dem angegebenen <see cref="Brush"/>-Objekt 
   ''' und dem angegebenen <see cref="Font"/>-Objekt.
   ''' </summary>
   ''' <param name="Text">Zu zeichnende Zeichenfolge.</param>
   ''' <param name="Font"><see cref="Font"/>, die das Textformat der Zeichenfolge definiert.</param>
   ''' <param name="Brush"><see cref="Brush"/>, der die Farbe und Struktur des gezeichneten Texts bestimmt.</param>
   ''' <param name="x">Die skalierte x-Koordinate der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <param name="y">Die skalierte x-Koordinate der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <remarks>Die Textgröße wird nicht skaliert.</remarks>
   Public Sub DrawString(Text As String, Font As Font, Brush As Brush, x As Single, y As Single)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawString(Text, Font, Brush, pts(0))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet die angegebene Textzeichenfolge an der angegebenen Position mit dem angegebenen <see cref="Brush"/>-Objekt 
   ''' und dem angegebenen <see cref="Font"/>-Objekt.
   ''' </summary>
   ''' <param name="Text">Zu zeichnende Zeichenfolge.</param>
   ''' <param name="Font"><see cref="Font"/>, die das Textformat der Zeichenfolge definiert.</param>
   ''' <param name="Brush"><see cref="Brush"/>, der die Farbe und Struktur des gezeichneten Texts bestimmt.</param>
   ''' <param name="x">Die skalierte x-Koordinate der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <param name="y">Die skalierte x-Koordinate der oberen linken Ecke des gezeichneten Texts.</param>
   ''' <remarks>Die Textgröße wird nicht skaliert.</remarks>
   Public Sub DrawString(Text As String, Font As Font, Brush As Brush, x As Double, y As Double)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(CSng(x), CSng(y))}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawString(Text, Font, Brush, pts(0))
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " DrawLine "
   ''' <summary>
   ''' Zeichnet eine verbindende Linie zwischen zwei Point-Strukturen.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Linie bestimmt.</param>
   ''' <param name="pt1">Point-Struktur, die den ersten zu verbindenden Punkt darstellt.</param>
   ''' <param name="pt2">Point-Struktur, die den zweiten zu verbindenden Punkt darstellt.</param>
   ''' <seealso cref="Graphics.DrawLine(Pen, Point, Point)"/>
   Public Sub DrawLine(Pen As Pen, pt1 As Point, pt2 As Point)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {pt1, pt2}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLine(Pen, pts(0), pts(1))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine verbindende Linie zwischen zwei PointF-Strukturen.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Linie bestimmt.</param>
   ''' <param name="pt1">Point-Struktur, die den ersten zu verbindenden Punkt darstellt.</param>
   ''' <param name="pt2">Point-Struktur, die den zweiten zu verbindenden Punkt darstellt.</param>
   ''' <seealso cref="Graphics.DrawLine(Pen, PointF, PointF)"/>
   Public Sub DrawLine(Pen As Pen, pt1 As PointF, pt2 As PointF)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {pt1, pt2}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLine(Pen, pts(0), pts(1))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine verbindende Linie zwischen den beiden durch Koordinatenpaare angegebenen Punkten.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Linie bestimmt.</param>
   ''' <param name="x1">Die x-Koordinate des ersten Punkts.</param>
   ''' <param name="y1">Die y-Koordinate des ersten Punkts.</param>
   ''' <param name="x2">Die x-Koordinate des zweiten Punkts.</param>
   ''' <param name="y2">Die y-Koordinate des zweiten Punkts.</param>
   ''' <seealso cref="Graphics.DrawLine(Pen, Integer, Integer, Integer, Integer)"/>
   Public Sub DrawLine(Pen As Pen, x1 As Integer, y1 As Integer, x2 As Integer, y2 As Integer)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New Point(x1, y1), New Point(x2, y2)}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLine(Pen, pts(0), pts(1))
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine verbindende Linie zwischen den beiden durch Koordinatenpaare angegebenen Punkten.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Linie bestimmt.</param>
   ''' <param name="x1">Die x-Koordinate des ersten Punkts.</param>
   ''' <param name="y1">Die y-Koordinate des ersten Punkts.</param>
   ''' <param name="x2">Die x-Koordinate des zweiten Punkts.</param>
   ''' <param name="y2">Die y-Koordinate des zweiten Punkts.</param>
   ''' <seealso cref="Graphics.DrawLine(Pen, Single, Single, Single, Single)"/>
   Public Sub DrawLine(Pen As Pen, x1 As Single, y1 As Single, x2 As Single, y2 As Single)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x1, y1), New PointF(x2, y2)}
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLine(Pen, pts(0), pts(1))
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " DrawLines "
   ''' <summary>
   ''' Zeichnet eine Reihe von Liniensegmenten, die ein Array von PointF-Strukturen verbinden.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Liniensegmente bestimmt.</param>
   ''' <param name="points">Array von PointF-Strukturen, die die zu verbindenden Punkte darstellen.</param>
   ''' <seealso cref="Graphics.DrawLines(Pen, PointF())"/>
   Public Sub DrawLines(Pen As Pen, points As PointF())
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As PointF() = DirectCast(points.Clone, PointF())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLines(Pen, pts)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine Reihe von Liniensegmenten, die ein Array von Point-Strukturen verbinden.
   ''' </summary>
   ''' <param name="Pen">Pen, der die Farbe, Breite und den Stil der Liniensegmente bestimmt.</param>
   ''' <param name="points">Array von Point-Strukturen, die die zu verbindenden Punkte darstellen.</param>
   ''' <seealso cref="Graphics.DrawLines(Pen, Point())"/>
   Public Sub DrawLines(Pen As Pen, points As Point())
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As Point() = DirectCast(points.Clone, Point())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawLines(Pen, pts)
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " DrawCurve "
   ''' <summary>
   ''' Zeichnet eine Cardinal-Splinekurve durch ein angegebenes Array von Point-Strukturen.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und Höhe der Kurve bestimmt.</param>
   ''' <param name="points">Array von Point-Strukturen, die die Splinekurve definieren.</param>
   ''' <seealso cref="Graphics.DrawCurve(Pen, Point())"/>
   Public Sub DrawCurve(pen As Pen, points As Point())
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As Point() = DirectCast(points.Clone, Point())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawCurve(pen, pts)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine Cardinal-Splinekurve durch ein angegebenes Array von PointF-Strukturen.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und Höhe der Kurve bestimmt.</param>
   ''' <param name="points">Array von PointF-Strukturen, die die Splinekurve definieren.</param>
   ''' <seealso cref="Graphics.DrawCurve(Pen, PointF())"/>
   Public Sub DrawCurve(pen As Pen, points As PointF())
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As PointF() = DirectCast(points.Clone, PointF())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawCurve(pen, pts)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet unter Verwendung der angegebenen Spannung eine Cardinal-Splinekurve durch ein angegebenes Array von Point-Strukturen.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und Höhe der Kurve bestimmt.</param>
   ''' <param name="points">Array von Point-Strukturen, die die Splinekurve definieren.</param>
   ''' <param name="tension">Ein Wert größer oder gleich 0,0F, der die Spannung der Kurve angibt. </param>
   ''' <seealso cref="Graphics.DrawCurve(Pen, Point(), Single)"/>
   Public Sub DrawCurve(pen As Pen, points As Point(), tension As Single)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As Point() = DirectCast(points.Clone, Point())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawCurve(pen, pts, tension)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet unter Verwendung der angegebenen Spannung eine Cardinal-Splinekurve durch ein angegebenes Array von PointF-Strukturen.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und Höhe der Kurve bestimmt.</param>
   ''' <param name="points">Array von PointF-Strukturen, die die Splinekurve definieren.</param>
   ''' <param name="tension">Ein Wert größer oder gleich 0,0F, der die Spannung der Kurve angibt. </param>
   ''' <seealso cref="Graphics.DrawCurve(Pen, PointF(), Single)"/>
   Public Sub DrawCurve(pen As Pen, points As PointF(), tension As Single)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts As PointF() = DirectCast(points.Clone, PointF())
      _TransformMatrix.TransformPoints(pts)
      _Graphics.DrawCurve(pen, pts, tension)
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " DrawRectangle "
   ''' <summary>
   ''' Zeichnet ein Rechteck, das von einer Rectangle-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Ein Pen, der die Farbe, Breite und den Stil des Rechtecks bestimmt.</param>
   ''' <param name="rect">Eine Rectangle-Struktur, die das zu zeichnende Rechteck darstellt.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawRectangle(Pen, Rectangle)"/>
   Public Sub DrawRectangle(pen As Pen, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawRectangle(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Zeichnet ein Rechteck, das durch ein Koordinatenpaar, eine Breiten- und eine Höhenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil des Rechtecks bestimmt.</param>
   ''' <param name="x">Die X-Koordinate der oberen linken Ecke des zu zeichnenden Rechtecks.</param>
   ''' <param name="y">Die Y-Koordinate der oberen linken Ecke des zu zeichnenden Rechtecks.</param>
   ''' <param name="width">Breite des zu zeichnenden Rechtecks.</param>
   ''' <param name="height">Höhe des zu zeichnenden Rechtecks.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawRectangle(Pen, Integer, Integer, Integer, Integer)"/>
   Public Sub DrawRectangle(pen As Pen, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New Point(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.DrawRectangle(pen, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet ein Rechteck, das von einer RectangleF-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Ein Pen, der die Farbe, Breite und den Stil des Rechtecks bestimmt.</param>
   ''' <param name="rect">Eine Rectangle-Struktur, die das zu zeichnende Rechteck darstellt.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawRectangle(Pen, Rectangle)"/>
   Public Sub DrawRectangle(pen As Pen, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawRectangle(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Zeichnet ein Rechteck, das durch ein Koordinatenpaar, eine Breiten- und eine Höhenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil des Rechtecks bestimmt.</param>
   ''' <param name="x">Die X-Koordinate der oberen linken Ecke des zu zeichnenden Rechtecks.</param>
   ''' <param name="y">Die Y-Koordinate der oberen linken Ecke des zu zeichnenden Rechtecks.</param>
   ''' <param name="width">Breite des zu zeichnenden Rechtecks.</param>
   ''' <param name="height">Höhe des zu zeichnenden Rechtecks.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawRectangle(Pen, Single, Single, Single, Single)"/>
   Public Sub DrawRectangle(pen As Pen, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.DrawRectangle(pen, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " FillRectangle "
   ''' <summary>
   ''' Füllt das Innere eines durch eine Rectangle-Struktur angegebenen Rechtecks.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die das zu füllende Rechteck darstellt.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, Rectangle)"/>
   Public Sub FillRectangle(brush As Brush, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillRectangle(brush, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Füllt das Innere eines Rechtecks, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="x">Die X-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="y">Die Y-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="width">Breite des zu füllenden Rechtecks.</param>
   ''' <param name="height">Höhe des zu füllenden Rechtecks.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, Integer, Integer, Integer, Integer)"/>
   Public Sub FillRectangle(brush As Brush, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New Point(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.FillRectangle(brush, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Füllt das Innere eines durch eine RectangleF-Struktur angegebenen Rechtecks.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die das zu füllende Rechteck darstellt.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, RectangleF)"/>
   Public Sub FillRectangle(brush As Brush, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillRectangle(brush, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Füllt das Innere eines Rechtecks, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="x">Die X-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="y">Die Y-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="width">Breite des zu füllenden Rechtecks.</param>
   ''' <param name="height">Höhe des zu füllenden Rechtecks.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, Single, Single, Single, Single)"/>
   Public Sub FillRectangle(brush As Brush, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.FillRectangle(brush, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub
#End Region

#Region " FillScaledRectangle "
   ''' <summary>
   ''' Füllt das Innere eines durch eine RectangleF-Struktur angegebenen Rechtecks.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die das zu füllende Rechteck darstellt.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung des Rechtecks wird transformiert. Breite und Höhe sind in skalierten maßen anzugeben.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, RectangleF)"/>
   Public Sub FillScaledRectangle(brush As Brush, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim rect1 = Transform(rect)
      '     Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).X - Width / 2, pts(0).X))
      '     Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).Y - Height / 2, pts(0).Y))
      _Graphics.FillRectangle(brush, rect1)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Füllt das Innere eines Rechtecks, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="x">Die X-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="y">Die Y-Koordinate der oberen linken Ecke des zu füllenden Rechtecks.</param>
   ''' <param name="width">Breite des zu füllenden Rechtecks.</param>
   ''' <param name="height">Höhe des zu füllenden Rechtecks.</param>
   ''' <param name="center">Gibt an, ob das Rechteck zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung und Größe des Rechtecks wird transformiert.</remarks>
   ''' <seealso cref="Graphics.FillRectangle(Brush, Single, Single, Single, Single)"/>
   Public Sub FillScaledRectangle(brush As Brush, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillScaledRectangle(brush, New RectangleF(x, y, width, height))
   End Sub
#End Region

#Region " DrawEllipse "
   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch Koordinaten für die obere linke Ecke des Rechtecks, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="x">Die x-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, Integer, Integer, Integer, Integer)"/>
   Public Sub DrawEllipse(pen As Pen, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New Point(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.DrawEllipse(pen, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch eine umschließende Rectangle-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die die Begrenzungen der Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, Rectangle)"/>
   Public Sub DrawEllipse(pen As Pen, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawEllipse(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch Koordinaten für die obere linke Ecke des Rechtecks, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="x">Die x-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite in Pixel des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe in Pixel des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, Single, Single, Single, Single)"/>
   Public Sub DrawEllipse(pen As Pen, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.DrawEllipse(pen, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch eine umschließende RectangleF-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die die Begrenzungen der Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, RectangleF)"/>
   Public Sub DrawEllipse(pen As Pen, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawEllipse(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub
#End Region

#Region " DrawScaledEllipse "
   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch Koordinaten für die obere linke Ecke des Rechtecks, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="x">Die x-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate (skaliertes Maß)d er oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   Public Sub DrawScaledEllipse(pen As Pen, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawScaledEllipse(pen, CSng(x), y, width, height, center)
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch eine umschließende Rectangle-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur (skalierte Maße), die die Begrenzungen der Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, Rectangle)"/>
   Public Sub DrawScaledEllipse(pen As Pen, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawScaledEllipse(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch Koordinaten für die obere linke Ecke des Rechtecks, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="x">Die x-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, Single, Single, Single, Single)"/>
   Public Sub DrawScaledEllipse(pen As Pen, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, x - width / 2, x))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, y + height / 2, y))

      Dim rect = New RectangleF(xt, yt, width, height)
      rect = Transform(rect)

      _Graphics.DrawEllipse(pen, rect)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Zeichnet eine Ellipse, die durch eine umschließende RectangleF-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="pen">Pen, der die Farbe, Breite und den Stil der Ellipse bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur (skalierte Maße), die die Begrenzungen der Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   ''' <seealso cref="Graphics.DrawEllipse(Pen, RectangleF)"/>
   Public Sub DrawScaledEllipse(pen As Pen, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      DrawEllipse(pen, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub
#End Region

#Region " FillEllipse "
   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt. </param>
   ''' <param name="x">Die x-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   Public Sub FillEllipse(brush As Brush, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New Point(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Integer = CInt(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.FillEllipse(brush, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch eine Rectangle-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die das umschließende Rechteck darstellt, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   Public Sub FillEllipse(brush As Brush, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillEllipse(brush, rect.X, rect.Y, rect.Width, rect.Height, ObjectAlignment.Center)
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt. </param>
   ''' <param name="x">Die x-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   Public Sub FillEllipse(brush As Brush, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim pts = {New PointF(x, y)}
      _TransformMatrix.TransformPoints(pts)
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).X - width / 2, pts(0).X))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, pts(0).Y - height / 2, pts(0).Y))
      _Graphics.FillEllipse(brush, xt, yt, width, height)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch eine RectangleF-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur, die das umschließende Rechteck darstellt, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse wird transformiert. Breite und Höhe behalten ihre ursprüngliche Größe.</remarks>
   Public Sub FillEllipse(brush As Brush, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillEllipse(brush, rect.X, rect.Y, rect.Width, rect.Height, ObjectAlignment.Center)
   End Sub
#End Region

#Region " FillScaledEllipse "
   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt. </param>
   ''' <param name="x">Die x-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate (skaliertes Maß) der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe (skaliertes Maß) des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   Public Sub FillScaledEllipse(brush As Brush, x As Integer, y As Integer, width As Integer, height As Integer, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillScaledEllipse(brush, CSng(x), y, width, height, center)
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch eine Rectangle-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur (skalierte Maße), die das umschließende Rechteck darstellt, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   Public Sub FillScaledEllipse(brush As Brush, rect As Rectangle, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillScaledEllipse(brush, rect.X, rect.Y, rect.Width, rect.Height, center)
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch ein Koordinatenpaar, eine Höhen- und eine Breitenangabe angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt. </param>
   ''' <param name="x">Die x-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="y">Die y-Koordinate der oberen linken Ecke des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="width">Breite des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="height">Höhe des umschließenden Rechtecks, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. X und Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   Public Sub FillScaledEllipse(brush As Brush, x As Single, y As Single, width As Single, height As Single, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      Dim tmpMatrix = _Graphics.Transform
      _Graphics.ResetTransform()
      Dim xt As Single = CSng(IIf(center = ObjectAlignment.Center, x - width / 2, x))
      Dim yt As Single = CSng(IIf(center = ObjectAlignment.Center, y + height / 2, y))

      Dim rect = New RectangleF(xt, yt, width, height)
      rect = Transform(rect)

      _Graphics.FillEllipse(brush, rect)
      _Graphics.Transform = tmpMatrix
   End Sub

   ''' <summary>
   ''' Füllt das Innere einer Ellipse, die durch ein umschließendes Rechteck definiert ist, das durch eine RectangleF-Struktur angegeben ist.
   ''' </summary>
   ''' <param name="brush">Brush, der die Merkmale des Füllvorgangs bestimmt.</param>
   ''' <param name="rect">Rectangle-Struktur (skalierte Maße), die das umschließende Rechteck darstellt, das die Ellipse definiert.</param>
   ''' <param name="center">Gibt an, ob die Ellipse zentiert dargestellt werden soll. rect.X und rect.Y sind dann die 
   ''' Mittelpunktskoordinaten (=Schnittpunkt der Diagonalen).</param>
   ''' <remarks>Der Ursprung der Ellipse, Breite und Höhe werden transformiert.</remarks>
   Public Sub FillScaledEllipse(brush As Brush, rect As RectangleF, Optional center As ObjectAlignment = ObjectAlignment.Standard)
      FillScaledEllipse(brush, rect.X, rect.Y, rect.Width, rect.Height, ObjectAlignment.Center)
   End Sub
#End Region
#End Region
End Class
